# install.packages('rattle')
# install.packages('rpart')
# install.packages('randomForest')

library(rattle)
library(rpart)
library(randomForest)

# Read Data and Partition Data into training and validation

p2 <- read.csv(file='example2_mccart.csv',header=T)

s1 <- sample(1:nrow(p2),nrow(p2)/2)
p2.train <- p2[s1,]
p2.test <- p2[-s1,]


# Fit tree in traiing data

d1 <- rpart(rho_alpha_diff ~ sk+sample_size+items+fact_dist+reliability,
      data=p2.train,control=rpart.control(cp=.001,minsplit=5))

fancyRpartPlot(d1)

# Get predicted values for the test data in training tree and compute MSE and R2

fitted <- predict(d1,newdata=p2.test)

mse.train <- mean((p2.train$rho_alpha_diff - predict(d1))^2)
r2.train <- cor(p2.train$rho_alpha_diff, predict(d1))^2
mse.test <- mean((p2.test$rho_alpha_diff - fitted)^2)
r2.test <- cor(p2.test$rho_alpha_diff, fitted)^2



## Cost complexity prunning
# Find cp where value is 1 S.E. from minimum

plotcp(d1)
prune.d1=prune(d1,cp=.0085)
fancyRpartPlot(prune.d1)

# Get predicted values for the test data in pruned tree and compute MSE and R2

fitted.prune <- predict(prune.d1,newdata=p2.test)

mse.prune.train <- mean((p2.train$rho_alpha_diff - predict(prune.d1))^2)
r2.prune.train <- cor(p2.train$rho_alpha_diff, predict(prune.d1))^2
mse.prune.test <- mean((p2.test$rho_alpha_diff - fitted.prune)^2)
r2.prune.test <- cor(p2.test$rho_alpha_diff, fitted.prune)^2



## RANDOM FORESTS (500 trees, 2 predictors at random)
bag1 <- randomForest(rho_alpha_diff~sample_size+reliability+items+fact_dist+sk,
     data=p2.train,mtry=2,importance=TRUE,ntree=500)

#obtain variable impotance values and variable importance plots
importance(bag1)
varImpPlot(bag1)

# Get predicted values for the test data from random forests and compute MSE and R2
fitted.bag.test <- predict(bag1,newdata=p2.test)

mse.train.bag <- mean((p2.train$rho_alpha_diff - predict(bag1)^2)
r2.train.bag <- cor(p2.train$rho_alpha_diff, predict(bag1))^2
mse.test.bag <- mean((p2.test$rho_alpha_diff - fitted.bag.test)^2)
r2.test.bag <- cor(p2.test$rho_alpha_diff, fitted.bag.test)^2




## BAGGING (500 trees, all predictors available)
bag2 <- randomForest(rho_alpha_diff~sample_size+reliability+items+fact_dist+sk,
        data=p2.train,mtry=5,importance=TRUE,ntree=500)

# Variable impotance values and plots
importance(bag2)
varImpPlot(bag2)


# Get predicted values for the test data from bagging and compute MSE and R2
fitted.test2 <- predict(bag2,newdata=p2.test)

mse.train.bag2 <- mean((p2.train$rho_alpha_diff - predict(bag2))^2)
r2.train.bag2 <- cor(p2.train$rho_alpha_diff, predict(bag2))^2
mse.test.bag2 <- mean((p2.test$rho_alpha_diff - fitted.test2)^2)
r2.test.bag2 <- cor(p2.test$rho_alpha_diff, fitted.test2)^2